package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.TokenUtil;
import com.xkrs.dao.*;
import com.xkrs.model.entity.*;
import com.xkrs.model.vo.MemberApplicationVo;
import com.xkrs.model.vo.MemberJoinVo;
import com.xkrs.service.MemberProjectService;
import com.xkrs.utils.DateTimeUtil;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author XinYi Song
 */
@Service
public class MemberProjectServiceImpl implements MemberProjectService {

    @Resource
    private MemberProjectDao memberProjectDao;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private ResponsibleProjectDao responsibleProjectDao;

    @Resource
    private ProjectOverviewDao projectOverviewDao;

    @Resource
    private WorkHourRecordDao workHourRecordDao;

    /**
     * 成员点击进入
     * @param projectNumber
     * @param token
     * @return
     */
    @Override
    public String memberJoin(String projectNumber, String token) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        ProjectOverview byProjectNumber = projectOverviewDao.findByProjectNumber(projectNumber);
        if("3".equals(byProjectNumber.getProjectStatus())){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"该项目正在审核，您还不能加入",locale);
        }
        MemberProject memberProjectNumber = memberProjectDao.findByMemberIdAndMemberProjectNumberAndApplicationType(sysUserEntity.getId(), projectNumber, "1");
        if(memberProjectNumber != null){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"您已经加入该项目",locale);
        }else {
            // 成员点击加入项目
            MemberProject memberProject = new MemberProject();
            memberProject.setMemberProjectNumber(projectNumber);
            memberProject.setMemberId(sysUserEntity.getId());
            memberProject.setApplicationTime(DateTimeUtil.dateTimeToString(LocalDateTime.now()));
            if("QT".equals(projectNumber.substring(0,2))){
                memberProject.setApplicationType("1");
                memberProjectDao.save(memberProject);
                WorkHourRecord workHourRecord = new WorkHourRecord();
                workHourRecord.setWorkContent("优化");
                workHourRecord.setWorkHour(0);
                workHourRecord.setUserId(sysUserEntity.getId());
                workHourRecord.setProjectNumber(projectNumber);
                workHourRecord.setColor("1");
                workHourRecord.setSubmitTime(DateTimeUtil.dateTimeToString(LocalDateTime.now()));
                workHourRecordDao.save(workHourRecord);
                return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"加入成功",locale);
            }
            memberProject.setApplicationType("0");
            memberProjectDao.save(memberProject);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"提交加入申请成功",locale);
        }
    }

    /**
     * 成员退出项目
     * @param projectNumber
     * @param token
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String memberDropOut(String projectNumber, String token) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        MemberProject memberProjectNumber = memberProjectDao.findByMemberIdAndMemberProjectNumberAndApplicationType(sysUserEntity.getId(), projectNumber, "1");
        if(memberProjectNumber == null){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"您已经进行该操作，请勿重复操作",locale);
        }
        ResponsibleProject projectNumber1 = responsibleProjectDao.findByPrincipalIdAndResponsibleProjectNumber(sysUserEntity.getId(), projectNumber);
        if(projectNumber1 != null){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"您已是负责人，请先移交权限再退出",locale);
        }
        memberProjectDao.deleteByMemberIdAndMemberProjectNumber(sysUserEntity.getId(),projectNumber);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"退出成功",locale);
    }

    /**
     * 成员被允许加入
     * @param projectNumber
     * @param userId
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void memberByAllow(String projectNumber,Integer userId) {
        WorkHourRecord workHourRecord = new WorkHourRecord();
        workHourRecord.setWorkContent("优化");
        workHourRecord.setWorkHour(0);
        workHourRecord.setUserId(userId);
        workHourRecord.setProjectNumber(projectNumber);
        workHourRecord.setColor("1");
        workHourRecord.setSubmitTime(DateTimeUtil.dateTimeToString(LocalDateTime.now()));
        workHourRecordDao.save(workHourRecord);
        memberProjectDao.updateApplicationType("1",projectNumber);
        memberProjectDao.updateJoinTime(DateTimeUtil.dateTimeToString(LocalDateTime.now()),projectNumber);
    }

    /**
     * 成员被拒绝加入
     * @param projectNumber
     * @param userId
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void memberByRefuse(String projectNumber,Integer userId) {
        memberProjectDao.updateApplicationType("2",projectNumber);
    }

    /**
     * 查询成员的加入申请
     * @param projectNumber
     * @return
     */
    @Override
    public List<MemberApplicationVo> selectMemberApplication(String projectNumber) {
        return memberProjectDao.selectMemberApplication(projectNumber);
    }

    /**
     * 查询加入的成员的信息
     * @param projectNumber
     * @return
     */
    @Override
    public List<MemberJoinVo> selectMemberJoin(String projectNumber) {
        return memberProjectDao.selectMemberJoin(projectNumber);
    }

    /**
     * 查询参与的项目的编号
     * @param userId
     * @return
     */
    @Override
    public List<String> selectProjectNumberByMemberId(Integer userId) {
        return memberProjectDao.selectProjectNumberByMemberId(userId,"1");
    }

    /**
     * 查询待审核项目的编号
     * @param userId
     * @return
     */
    @Override
    public List<String> selectProjectNumberByMemberIds(Integer userId) {
        return memberProjectDao.selectProjectNumberByMemberId(userId,"0");
    }


}
