package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.TokenUtil;
import com.xkrs.dao.*;
import com.xkrs.model.entity.ProjectOverview;
import com.xkrs.model.entity.ResponsibleProject;
import com.xkrs.model.entity.SysUserEntity;
import com.xkrs.model.entity.WorkHourRecord;
import com.xkrs.model.vo.ApplicationProjectVo;
import com.xkrs.service.ResponsibleProjectService;
import com.xkrs.utils.DateTimeUtil;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author XinYi Song
 */
@Service
public class ResponsibleProjectServiceImpl implements ResponsibleProjectService {

    @Resource
    private ResponsibleProjectDao responsibleProjectDao;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private ProjectOverviewDao projectOverviewDao;

    @Resource
    private MemberProjectDao memberProjectDao;

    @Resource
    private WorkHourRecordDao workHourRecordDao;
    /**
     * 负责人点击开始
     * @param responsibleProjectNumber
     * @param token
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String operateResponsible(String responsibleProjectNumber, String token) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        ProjectOverview byProjectNumber = projectOverviewDao.findByProjectNumber(responsibleProjectNumber);
        if("3".equals(byProjectNumber.getProjectStatus())){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"该项目正在审核",locale);
        }
        if(!"0".equals(byProjectNumber.getProjectStatus())){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"该项目正在执行中",locale);
        }
        // 负责人点击开始，项目提交申请
        ResponsibleProject responsibleProject1 = new ResponsibleProject();
        responsibleProject1.setPrincipalId(sysUserEntity.getId());
        responsibleProject1.setResponsibleProjectNumber(responsibleProjectNumber);
        responsibleProject1.setApplicationTime(DateTimeUtil.dateTimeToString(LocalDateTime.now()));
        responsibleProject1.setAuditType("0");
        projectOverviewDao.updateProjectStatus("3",responsibleProjectNumber);
        responsibleProjectDao.save(responsibleProject1);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"操作成功",locale);
    }

    /**
     * 负责人对项目进行重新开始的操作
     * @param ProjectNumber
     * @param token
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String operateReStart(String ProjectNumber, String token) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        projectOverviewDao.updateProjectStatus("1",ProjectNumber);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"操作成功",locale);
    }

    /**
     * 负责人对项目进行结束的操作
     * @param ProjectNumber
     * @param token
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String operateOver(String ProjectNumber, String token) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        projectOverviewDao.updateProjectStatus("2",ProjectNumber);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"操作成功",locale);
    }

    /**
     * 查询负责人申请的相关信息
     * @param projectNumber
     * @return
     */
    @Override
    public ApplicationProjectVo selectApplicationInformation(String projectNumber) {
        ApplicationProjectVo applicationProjectVo = responsibleProjectDao.selectApplicationInformation(projectNumber);
        return applicationProjectVo;
    }

    /**
     * 管理员允许负责人申请通过
     * @param projectNumber
     * @param userId
     * @return
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void adminAllow(String projectNumber,Integer userId) {
        WorkHourRecord workHourRecord = new WorkHourRecord();
        workHourRecord.setWorkContent("优化");
        workHourRecord.setWorkHour(0);
        workHourRecord.setUserId(userId);
        workHourRecord.setProjectNumber(projectNumber);
        workHourRecord.setColor("1");
        workHourRecord.setSubmitTime(DateTimeUtil.dateTimeToString(LocalDateTime.now()));
        workHourRecordDao.save(workHourRecord);
        // 修改中间表的申请状态为1（表示审核通过）
        responsibleProjectDao.updateAuditType("1",projectNumber);
        responsibleProjectDao.updateStartTime(DateTimeUtil.dateTimeToString(LocalDateTime.now()),projectNumber);
        // 修改项目表中的项目状态为1（表示填报中）
        projectOverviewDao.updateProjectStatus("1",projectNumber);
    }

    /**
     * 管理员拒绝负责人的申请
     * @param projectNumber
     * @param userId
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void adminRefuse(String projectNumber,Integer userId) {
        // 修改中间表的申请状态为2（表示负责人提交的申请被拒绝）
        responsibleProjectDao.updateAuditType("2",projectNumber);
        // 修改项目表中的状态为0（表示负责人提交的申请被拒绝，项目变成未开始状态）
        projectOverviewDao.updateProjectStatus("0",projectNumber);
    }

    /**
     * 负责人移除成员
     * @param memberId
     * @param projectNumber
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void removeMember(Integer memberId, String projectNumber) {
        memberProjectDao.deleteByMemberIdAndMemberProjectNumber(memberId,projectNumber);
    }

    /**
     * 负责人移交权限
     * @param memberId
     * @param projectNumber
     * @param userId
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public void transferPermissions(Integer memberId, String projectNumber, Integer userId) {
        // userId负责人的id  memberId成员的id
        memberProjectDao.updateMemberIdByProjectNumber(userId,projectNumber,memberId);
        responsibleProjectDao.updatePrincipalIdByProjectNumber(memberId,projectNumber);
    }

    /**
     * 查询管理的项目的编号
     * @param userId
     * @return
     */
    @Override
    public List<String> selectProjectNumberByPrincipalId(Integer userId) {
        return responsibleProjectDao.selectProjectNumberByPrincipalId(userId,"1");
    }

    /**
     * 查询正在审核的项目的编号
     * @param userId
     * @return
     */
    @Override
    public List<String> selectProjectNumberByPrincipalIds(Integer userId) {
        return responsibleProjectDao.selectProjectNumberByPrincipalId(userId,"0");
    }
}
